
# python
from urllib import urlopen
from urlparse import urlparse, urlunparse
from httplib import HTTPConnection, HTTPSConnection,\
     HTTPException, BadStatusLine
from socket import error as socket_error
import sys, errno

# zope
import zLOG

# local
from ReferenceChecker import ReferenceInfo


USER_AGENT = 'Silva Reference Checker (0.1) (contact: http://zope.org/Members/camil7/SilvaReferenceChecker/)'

class dummy_status:

    def __init__(self,status,reason=''):
        self.status = status
        self.reason=reason

class LinkCheck:

    def __init__(self, ref):
        self.ref = ref

    def __call__(self):
        # stupid delegation;
        # I just don't like to have too much code in a __magic__ method
        try:
            self.checkUrl(self.ref)
        except:
            # raising exceptions out of the check method is not allowed
            zLOG.LOG('Silva Reference Checker',100,
                     'bad exception while checking url %s' % self.ref.get_target(), '',
                     sys.exc_info())

    def __str__(self):
        " string representation for debugging "
        return 'checking '+self.ref.get_target()


    def _get_conn(self,scheme, host):
        if scheme == 'https':
            return HTTPSConnection(host)
        else:
            return HTTPConnection(host)


    def checkUrl(self, ref):
        status = ReferenceInfo.STATUS_MISSING
        url = ref.get_target()
        try:
            scheme, host, path, param, query, fragment = urlparse(url)

            if scheme in ('http', 'https'):
                
                path = urlunparse( ('','',path, param, query, '') )
		if not path:
		    path = '/'
                
                headers = {
                    'User-Agent':USER_AGENT,
                    }

                conn = self._get_conn(scheme, host)
                # this should work with python 2.3.2
                #conn.connect()                
                #conn.sock.set_timeout( <some reasonable value here> )
                conn.request("HEAD", path, headers=headers)
                try:
                    r1 = conn.getresponse()
                except BadStatusLine: # ugh
                    r1 = dummy_status(501)
                if r1.status in (400, 403, 405, 406, 500, 501) :
                    # HACK: some servers do not support HEAD, try a GET:
                    # (note the server should respond with a 501, but
                    # usually servers tend to blame the client with a 40x ...)
                    conn.close()
                    conn = self._get_conn(scheme, host)
                    conn.request("GET", path, headers=headers)
                    r1 = conn.getresponse()
                
                if r1.status < 400:
                    if r1.status == 200:
                        status = ReferenceInfo.STATUS_OK
                    else:
                        status = ReferenceInfo.STATUS_WARNING
                else:
                    zLOG.LOG('Silva Reference Checker',-300,
                             'url %s got problem %s' %(url, r1.status) )
                    status = ReferenceInfo.STATUS_BROKEN
                ref.message='%s %s' % (r1.status, r1.reason)
                conn.close()
            else:
                # no http(s):
                # just try to open the url, no status code checks
                urlopen(url)
                status = ReferenceInfo.STATUS_OK
        except BadStatusLine, e:
	    # problem: the str(e) method is broken for this exception
	    # (at least in python 2.1.3)
            zLOG.LOG('Silva Reference Checker',-200,
                     'bad status line',
                     '%s\n' % (url,) )
            status = ReferenceInfo.STATUS_BROKEN
            ref.message = 'Status code unreadable'            
        except HTTPException, e:
            # hm. this could be a programming error or a server problem
            # tell user its the other sides fault ;-)
            zLOG.LOG('Silva Reference Checker',-300,
                     'http exception [[%s]]' % (e,),
                     '%s\n' % (url,) )
            status = ReferenceInfo.STATUS_BROKEN
            ref.message = 'status code unreadable [%s]' % str(e)
        except socket_error, e:
            zLOG.LOG('Silva Reference Checker',-300,
                     'socket error [[%s]] when retrieving url' % (e,) ,
                     '%s\n' % (url,))
            try:
                code, message = e.args
                if type(code) != type(0): raise ValueError
            except ValueError:
                # hm, unpacking the exceptions argument failed -> no error code
                code = -1
                message = str(e)
            except AttributeError:
                # ugh, we do not even have an "args" sometimes :/
                code = -1
                message = str(e)
            
            # ususally users do not know about socket error codes
            # actually me does not, thus dropped ;-)
            ref.message = message
            if code == errno.ETIMEDOUT:
                status = ReferenceInfo.STATUS_TIMEOUT
                   
        except:
            # conclude that all unspecific exceptions are failures
            zLOG.LOG('Silva Reference Checker',-100,
                     'unknown error when retrieving url',
                     '%s\n' % url, sys.exc_info() )
            ref.message = str(sys.exc_info()[1])

        ref.status = status
