import unittest
import Zope

from DateTime import DateTime
from Products.ParsedXML.ParsedXML import ParsedXML
from Products.Silva.tests.test_SilvaObject import hack_create_user
from Products.Silva.Ghost import GhostVersion
from Testing import makerequest

import Products.SilvaReferenceChecker.ReferenceChecker
from os.path import dirname, join, split

dir, _  = split(Products.SilvaReferenceChecker.ReferenceChecker.__file__)
pfeil_rot_gif = join(dir,'tests/pfeil_rot.gif')
README = join(dir,'README.txt')


def add_helper(context, type, id, title):
    factory = getattr(context.manage_addProduct['Silva'], 'manage_add%s' % type)
    factory(id, title)
    return getattr(context, id)

class ReferenceCheckerTestCase(unittest.TestCase):

    def setUp(self):

        get_transaction().begin()
        self.connection = Zope.DB.open()
        try:
            self.root = makerequest.makerequest(self.connection.root()['Application'])
            self.root.REQUEST['URL1'] = '' # to fix problem with the permission check
            hack_create_user(self.root)
                
            self.root.manage_addProduct['Silva'].manage_addRoot('root', 'Silva Root')
            self.sroot = self.root.root
            # this automatically dreates an "index" which we dont want to check here
            self.sroot.get_default().close_version()
            
            # example image
            self.sroot.manage_addFolder('images',"Images dummy")
            self.sroot.images.manage_addImage('pfeil_rot.gif', open(pfeil_rot_gif, 'rb'),
                                              content_type='image/gif')

            # XXX does this still work with silva 0.9.3 ?
            self.editorsupport = self.sroot.service_editorsupport
            
            self.rc = self.sroot.manage_addProduct['SilvaReferenceChecker'].manage_addReferenceChecker()
            self.assertEqual(self.rc.meta_type, 'Silva Reference Checker')
            # self.rc._default_abs_root = ReferenceInfo.PATH_SILVA_ROOT

            # Doc, Folder
            self.doc1 = add_helper(self.sroot, 'Document', 'test_doc1','Test doc')

            addFolder = self.sroot.manage_addProduct['Silva'].manage_addFolder
            addFolder('test_folder', 'Test Folder')
            self.folder = self.sroot.test_folder
            addFolder('test_folder_foo', 'Test Folder for Weird Acquisition')
            self.folder_foo = self.sroot.test_folder_foo

            self.doc2 = add_helper(self.folder, 'Document', 'test_doc2','Test doc 2')
            self.doc3 = add_helper(self.folder_foo, 'Document','test_doc3','Test doc 3')
            self.doc4 = add_helper(self.folder_foo, 'Document', 'test_doc4','Test doc 4')

            # Image
            self.folder.manage_addProduct['Silva'].manage_addImage('silva_image1','Silva Image1',
                                                                   open(pfeil_rot_gif, 'rb'))
            self.silva_image1 = self.folder.silva_image1

            # Ghost 1 correct 3 wrong type
            add_ghost = self.folder_foo.manage_addProduct['Silva'].manage_addGhost
            add_ghost('test_ghost1', '/root/test_folder/test_doc2')
            self.ghost1 = self.folder_foo.test_ghost1
            add_ghost('test_ghost2','/root/test_folder_foo/test_ghost1')
            self.ghost2 = self.folder_foo.test_ghost2
            add_ghost('test_ghost3','/root/test_folder')
            self.ghost3 = self.folder_foo.test_ghost3
            add_ghost('test_ghost4', '/root/test_folder/silva_image1')
            self.ghost4 = self.folder_foo.test_ghost4
            
            # manage_addFile() complains because of unvalid file format
            self.folder.manage_addProduct['Silva'].manage_addFile('silva_file1','Silva File1',
                                                                   open(README, 'rb'))
            self.silva_file1 = self.folder.silva_file1

            # Check test objects
            self.assertEqual(self.sroot.id, 'root')
            self.assertEqual(self.sroot.meta_type, 'Silva Root')
            self.assertEqual(self.doc1.getPhysicalPath(), ('', 'root', 'test_doc1'))

            self.assertEqual(self.folder.getPhysicalPath(), ('', 'root', 'test_folder'))
            self.assertEqual(self.doc2.getPhysicalPath(), ('', 'root', 'test_folder', 'test_doc2') )

            self.assertEqual(self.folder_foo.getPhysicalPath(), ('', 'root', 'test_folder_foo'))
            self.assertEqual(self.doc3.getPhysicalPath(), ('', 'root', 'test_folder_foo', 'test_doc3') )
            self.assertEqual(self.doc4.getPhysicalPath(), ('', 'root', 'test_folder_foo', 'test_doc4') )

            self.assertEqual(self.silva_image1.getPhysicalPath(), ('', 'root', 'test_folder', 'silva_image1') )

            self.assertEqual(self.ghost1.getPhysicalPath(), ('', 'root', 'test_folder_foo', 'test_ghost1') )
            self.assertEqual(self.ghost2.getPhysicalPath(), ('', 'root', 'test_folder_foo', 'test_ghost2') )
            self.assertEqual(self.ghost3.getPhysicalPath(), ('', 'root', 'test_folder_foo', 'test_ghost3') )
            self.assertEqual(self.ghost4.getPhysicalPath(), ('', 'root', 'test_folder_foo', 'test_ghost4') )


            # Publish folder
            self.folder.get_default().set_unapproved_version_publication_datetime(DateTime()-1)
            self.folder.get_default().approve_version()
            self.folder_foo.get_default().set_unapproved_version_publication_datetime(DateTime()-1)
            self.folder_foo.get_default().approve_version()
            
            # Fill/publish documents
            self._fillAndPublishDoc1()
            self._fillAndPublishDoc2()
            self._fillAndPublishDoc3()
            # self.doc4 is unpublished!
            self._publish(self.ghost1)
            self._publish(self.ghost2)
            self._publish(self.ghost3)
            self._publish(self.ghost4)
        except:
            self.tearDown()
            raise

    def tearDown(self):
        get_transaction().abort()
        self.connection.close()

    def _publish(self, doc):
        assertTrue = self.assert_
        assertFalse = self.failIf
        assertFalse(doc.get_viewable())
        doc.set_unapproved_version_publication_datetime(DateTime()-1)
        doc.approve_version()
        assertTrue(doc.get_viewable())
        

    def _fillAndPublishDoc1(self):
        assertTrue = self.assert_
        assertFalse = self.failIf

        # The first three references are ok.
        # Then there is a case of
        # - unpublished reference,
        # - weird acquisition,
        # - wrong type, and
        # - missing
        # Total: 7
        xml = """\
        <doc>
        <p>Downloads</p>
        <p><link url="/test_folder/test_doc2">Ok</link></p>
        <image link="/test_folder/test_doc2" path="/root/test_folder/silva_image1"></image>
        <p><link url="/test_folder_foo/test_doc4">Unpublished</link></p>
        <p><link url="/test_folder/test_folder_foo/test_doc3">Weird Acquisition</link></p>
        <image path="/root/test_folder/test_doc2"></image>
        <p><link url="/test_folder_foo/test_doc2">Broken</link></p>
        </doc>
        """

        getattr(self.doc1,'0').content = ParsedXML('content', unicode(xml, 'latin_1'))
        # publish the document
        self._publish(self.doc1)

    def _fillAndPublishDoc2(self):
        assertTrue = self.assert_
        assertFalse = self.failIf
        doc = getattr(self.doc2,'0').content.documentElement
        node = doc.createElement('p')
        node.appendChild(doc.createTextNode(''))
        doc.appendChild(node)

        # put some content into paragraph node including links
        # The first two internal references are ok.
        # The first external reference is missing, the second is ok :-)
        # Then there is a missing internal reference and a mailto reference.
        # Total: 6
        input = """This is a helper text with ((a ++relative++ link|test_doc2)),
        An ((absolute link|/test_doc1|)) and
        a ((relative|../test_doc1)) link to the same document.
        A ((broken external link|http://foo.bar/test_doc1|popup)).
        An external link that usually works: ((zlb|http://www.zlb.de/)).
        A ((broken internal link|/to_nowhere/doc883)).
        A ((special link|mailto:someone@somewhere.net)) at the end."""
        self.editorsupport.replace_text(node, input)

        # publish the document
        self._publish(self.doc2)

    def _fillAndPublishDoc3(self):
        assertTrue = self.assert_
        assertFalse = self.failIf
        xml = """\
        <doc>
        <p>Nothing</p>
        </doc>
        """
        getattr(self.doc3,'0').content = ParsedXML('content', unicode(xml, 'latin_1'))
        # publish the document
        self._publish(self.doc3)

    def pr(self, refs):
        # print references: for debugging
        for r in refs: print r
        
    def testAInternalLinkBasics(self):
        assertEquals = self.assertEquals
        assertTrue = self.assert_
        assertFalse = self.failIf
        #print '2'*80
        rc = self.rc
        # collect references
        references = []
        rc._get_references_helper(rc.aq_inner.aq_parent, references)
        #self.pr(references)
        assertEquals(18, len(references))

        rc._check_internal_references()
        references = rc._internal_references._list
        #self.pr(references)

        assertEquals(15, len([r for r in references if r.ref_type==r.REF_INTERN]) )
        assertEquals( 2, len([r for r in references if r.restriction==r.RESTRICT_IMAGE]) )

        assertEquals(3, len([r for r in references if r.status==r.STATUS_WRONG_TYPE]) )
        assertEquals(1, len([r for r in references if r.status==r.STATUS_UNPUBLISHED]) )
        assertEquals(2, len([r for r in references if r.status==r.STATUS_MISSING]) )
        assertEquals(1, len([r for r in references if r.status==r.STATUS_WEIRD_ACQUISITION]) )
        assertEquals(7, len(rc.get_broken_internal_references()) )
        
    def testExternalLinkBasics(self):
        assertEquals = self.assertEquals
        assertTrue = self.assert_
        assertFalse = self.failIf
        rc = self.rc

        # collect references
        references = []
        rc._get_references_helper(rc.aq_inner.aq_parent, references)
        # self.pr(references)
        # determine type
        rc._determine_reference_types(references)
        assertEquals(len([r for r in references if r.ref_type==r.REF_EXTERN]), 2)
        assertEquals(len([r for r in references if r.ref_type==r.REF_MAILTO]), 1)
        # check
        rc._sleep_after_external_checks = 5.0 # default is 30
        rc._check_external_references()
        assertEquals(len(rc.get_broken_external_references()), 1)
        references = rc._external_references._list
        #self.pr(references)
        

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ReferenceCheckerTestCase, 'test'))
    return suite        

def main():
    unittest.TextTestRunner(verbosity=1).run(test_suite())

if __name__ == '__main__':
    main()
